/*
 * Raima Database Manager
 *
 * Copyright (c) 2019 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/*! \file
 * \brief Source code for the RDM core06 example
 */
#include <stdio.h>
#include <string.h>
#include "print_error.h"
#include "rdm.h"

/* Generated \c struct and \c typedef definitions to be used with the RDM APIs */
#include "core06_db_structs.h"

/* Generated catalog definition to be used with the RDM rdm_dbSetCatalog() API */
#include "core06_db_cat.h"

/*!
 * \mainpage Core06 Popcorn Example
 *
 * Each time you run this example:
 * \li The database is initialized (all existing contents removed)
 * \li Several new artists are added to the database
 * \li For each artist a couple albums are added to the database
 * \li For each album a tracks are added to the database
 * \li A list of each artist, their albums, and the album's tracks is
 *      displayed
 *
 * \par Table of Contents
 *
 * - \subpage hDB
 * - \subpage hRDMfunc
 * - \subpage hPGMfunc
 *
 * For additional information, please refer to the product documentation at http://docs.raima.com/.
 *
 * \page hDB Database Schema
 *
 * \par Database Schema Definition
 *
 * The DDL (Database Definition Language) specification for the database used
 * in this example is located in the file \c core06_db.sdl.
 *
 * \include core06_db.sdl
 *
 * The schema was compiled using the RDM rdm-compile utility with the -s option to
 * generate C-structures for interfacing with the database.
 *
 * \code rdm-compile -s core06_db.sdl \endcode
 *
 * \page hRDMfunc RDM Functions Used
 * The following RDM Functions are illustrated:
 * \li rdm_rdmAllocDatabase/rdm_dbFree
 * \li rdm_dbSetCatalog
 * \li rdm_dbOpen/rdm_dbClose
 * \li rdm_dbInsertRow
 * \li rdm_dbAllocCursor/rdm_cursorFree
 * \li rdm_dbGetRows
 * \li rdm_cursorMoveToFirst/rdm_cursorMoveToNext
 * \li rdm_cursorReadRow
 *
 * They are fully documented in http://docs.raima.com/.
 *
 * Because the database is opened in exclusive mode lock and transactions
 * are not required.  Future core examples will use shared mode and will
 * demonstrate the use of transactions and locks.
 *
 * Each of these functions returns an integer status code, where the
 * value sOKAY indicates a successful call.
 *
 * The actual database will be stored in a directory named 'core06.rdm' in the
 * project directory.
 *
 * \page hPGMfunc Program Functions
 * \li openEmptyDatabase() - \copybrief openEmptyDatabase
 * \li cleanup() - \copybrief cleanup
 * \li insertArtist() - \copybrief insertArtist
 * \li readAllAlbums() - \copybrief readAllAlbums
 * \li main() - \copybrief main
 */

/*! \brief Initialize the RDM runtime library for use in the core06 example
 *
 * This function initializes the RDM Transactional File Server (TFS) to use
 * the EMBEDED TFS implementation.  It also allocates a database handle
 * and opens the "core06" database in exclusive mode.
 * Exclusve mode does not require database locks or transactions.
 *
 * @return Returns RDM_RETCODE status code (sOKAY if successful)
 */
RDM_RETCODE openEmptyDatabase (RDM_DB *pDB) /**< [out] Pointer to the RDM database handle */
{
    RDM_RETCODE rc = sOKAY;

    rc = rdm_rdmAllocDatabase (pDB);
    print_error (rc);

    if (rc == sOKAY)
    {
        /* Override the default DURABLE setting to CONSISTENT */
        rc = rdm_dbSetOptions (*pDB, "durability=consistent");
        print_error (rc);

        /* If we have a DB handle ... */
        rc = rdm_dbSetCatalog (*pDB, core06_db_cat);
        print_error (rc);

        if (rc == sOKAY)
        {
            rc = rdm_dbOpen (*pDB, "core06", RDM_OPEN_EXCLUSIVE);
            print_error (rc);
        }
        if (rc == sOKAY)
        {
            rc = rdm_dbDeleteAllRowsFromDatabase (*pDB);
            print_error (rc);
        }

        /* If any of the above DB operations failed, free the DB handle */
        if (rc != sOKAY)
        {
            rdm_dbFree (*pDB);
        }
    }
    return rc;
}

/*! \brief Cleanup the RDM runtime library
 *
 * This functions closes all open databases and cleans up the RDM runtime task
 * used in the core06 example.
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
void cleanup (RDM_DB hDB) /**< [out] Pointer to the DB handle to be terminated */
{
    /* close the database */
    rdm_dbClose (hDB);

    /* free the database handle */
    rdm_dbFree (hDB);
}

/*! \brief Insert an artis
 *
 * This function adds one more row to the ARTIST table in the core03 database.
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
RDM_RETCODE insertArtist (RDM_DB hDB,             /**< [in] Database handle to open database */
                          const char *artistName) /**< [in] Artist name to insert */
{
    RDM_RETCODE rc;
    ARTIST artist_rec;

    artist_rec.ARTISTID = 0;
    strncpy (artist_rec.NAME, artistName, sizeof (artist_rec.NAME));
    rc = rdm_dbInsertRow (hDB, TABLE_ARTIST, &artist_rec, sizeof (artist_rec), NULL);
    print_error (rc);

    return rc;
}

/*! \brief Insert an artist and all of the albums associated with the artist
 *
 * This function adds one more row to the ARTIST table in the core03 database and
 * one to many rows to the ALBUM table associated with the given artist.
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
RDM_RETCODE insertAlbum (RDM_DB hDB,             /**< [in] Database handle to open database */
                         const char *artistName, /**< [in] Name of the artist */
                         const char *albumTitle, /**< [in] Artist name to insert */
                         const char **trackList, /**< [in] List of album names for the artist */
                         size_t listSize)        /**< [in] Number of album names in the list */
{
    RDM_RETCODE rc;
    ALBUM album_rec;
    RDM_CURSOR albumCursor = NULL;
    RDM_CURSOR artistCursor = NULL;
    ARTIST_NAME_KEY artistKey;

    strncpy (artistKey.NAME, artistName, sizeof (artistKey.NAME));
    rc = rdm_dbGetRowsByKeyAtKey (hDB, KEY_ARTIST_NAME, &artistKey, sizeof (artistKey), &artistCursor);
    print_error (rc);

    if (rc == sOKAY)
    {
        strncpy (album_rec.TITLE, albumTitle, sizeof (album_rec.TITLE));
        album_rec.ALBUMID = 0;
        album_rec._ARTISTID_has_value = RDM_FALSE; /* Mark the row as unlinked */
        rc = rdm_dbInsertRow (hDB, TABLE_ALBUM, &album_rec, sizeof (album_rec), &albumCursor);
        print_error (rc);
    }

    if (rc == sOKAY)
    {
        /* associate the added row to the artist */
        rc = rdm_cursorLinkRow (albumCursor, REF_ALBUM_ARTISTID, artistCursor);
        print_error (rc);
        if (rc == sOKAY)
        {
            int ii;

            for (ii = 0; ii < (int) listSize; ii++)
            {
                TRACK track_rec;
                RDM_CURSOR trackCursor = NULL;

                strncpy (track_rec.TITLE, trackList[ii], sizeof (track_rec.TITLE));
                track_rec._ALBUMID_has_value = RDM_FALSE; /* Mark the row as unlinked */
                rc = rdm_dbInsertRow (hDB, TABLE_TRACK, &track_rec, sizeof (track_rec), &trackCursor);
                print_error (rc);

                if (rc == sOKAY)
                {
                    /* associate the added row to the artist */
                    rc = rdm_cursorLinkRow (trackCursor, REF_TRACK_ALBUMID, albumCursor);
                    print_error (rc);

                    rdm_cursorFree (trackCursor);
                }
            }
            rdm_cursorFree (albumCursor);
        }
    }
    return rc;
}

/*! \brief Read all the artists and the associated albums and tracks
 *
 * This function reads and displays each row from the ARTIST table in the core06 database
 * and the ALBUMS and TRACKS associated with the artist.
 *
 * @return Returns an RDM_RETCODE code (sOKAY if successful)
 */
RDM_RETCODE readAllAlbums (RDM_DB hDB) /**< [in] Database handle to open database */
{
    RDM_RETCODE rc;
    RDM_CURSOR artistCursor = NULL;

    /* Loop through each artist record and diplay the albums/tracks it
       is associated with
     */
    printf ("\nList all artists and their albums/tracks\n");

    /* read the ARTIST table in table order */
    rc = rdm_dbGetRowsByKey (hDB, KEY_ARTIST_NAME, &artistCursor);
    print_error (rc);

    if (rc == sOKAY)
    {
        for (rc = rdm_cursorMoveToFirst (artistCursor); rc == sOKAY; rc = rdm_cursorMoveToNext (artistCursor))
        {
            RDM_CURSOR albumCursor = NULL;
            char Artist[RDM_COLUMN_SIZE (ARTIST, NAME)];

            /* We found an artist row, read the contents */
            rc = rdm_cursorReadColumn (artistCursor, COL_ARTIST_NAME, Artist, sizeof (Artist), NULL);
            print_error (rc);

            if (rc == sOKAY)
            {
                printf ("\nArtist: %s\n", Artist);
                /* read the album rows associated with the artist*/
                rc = rdm_cursorGetMemberRows (artistCursor, REF_ALBUM_ARTISTID, &albumCursor);
                print_error (rc);

                if (rc == sOKAY)
                {
                    for (rc = rdm_cursorMoveToFirst (albumCursor); rc == sOKAY; rc = rdm_cursorMoveToNext (albumCursor))
                    {
                        RDM_CURSOR trackCursor = NULL;
                        char Title[RDM_COLUMN_SIZE (ALBUM, TITLE)];

                        /* We found an album row, read the contents*/
                        rc = rdm_cursorReadColumn (albumCursor, COL_ALBUM_TITLE, Title, sizeof (Title), NULL);
                        print_error (rc);

                        if (rc == sOKAY)
                        {
                            printf ("\n\tAlbum: %s\n", Title);
                            rc = rdm_cursorGetMemberRows (albumCursor, REF_TRACK_ALBUMID, &trackCursor);
                            print_error (rc);

                            if (rc == sOKAY)
                            {
                                for (rc = rdm_cursorMoveToFirst (trackCursor); rc == sOKAY;
                                     rc = rdm_cursorMoveToNext (trackCursor))
                                {
                                    char Track[RDM_COLUMN_SIZE (TRACK, TITLE)];

                                    /* We found an album row, read the contents*/
                                    rc = rdm_cursorReadColumn (trackCursor, COL_TRACK_TITLE, Track, sizeof (Track),
                                                               NULL);
                                    print_error (rc);

                                    if (rc == sOKAY)
                                    {
                                        printf ("\t\t%s\n", Track);
                                    }
                                }

                                rdm_cursorFree (trackCursor);
                            }
                        }
                    }
                    rdm_cursorFree (albumCursor);
                }
            }
        }
        rdm_cursorFree (artistCursor);
        /* We expect rc to be sENDOFCURSOR when we break out of the loop */
        if (rc == sENDOFCURSOR)
        {
            rc = sOKAY; /* change status to sOKAY because sENDOFCURSOR was expected. */
        }
        else
        {
            print_error (rc);
        }
    }
    return rc;
}

/* Track data to insert */
static const char *the_doors_tracks[] = {"Break on Through (To the Other Side)",
                                         "Soul Kitchen",
                                         "The Crystal Ship",
                                         "Twentieth Century Fox",
                                         "Alabama Song (Whisky Bar)",
                                         "Light My Fire",
                                         "Back Door Man",
                                         "I Looked at You",
                                         "End of the Night",
                                         "Take It as It Comes",
                                         "The End"};

static const char *strange_days_tracks[] = {
    "Strange Days",         "You're Lost Little Girl", "Love Me Two Times",
    "Unhappy Girl",         "Horse Latitudes",         "Moonlight Drive",
    "People Are Strange",   "My Eyes Have Seen You",   "I Can't See Your Face in My Mind",
    "When the Music's Over"};

static const char *waiting_tracks[] = {"Hello, I Love You",    "Love Street",     "Not to Touch the Earth",
                                       "Summer's Almost Gone", "Wintertime Love", "The Unknown Soldier",
                                       "Spanish Caravan",      "My Wild Love",    "We Could Be So Good Together",
                                       "Yes, the River Knows", "Five to One"};

static const char *the_rolling_stones_tracks[] = {"Not Fade Away",
                                                  "Route 66",
                                                  "I Just Want to Make Love to You",
                                                  "Honest I Do",
                                                  "Now I've Got a Witness",
                                                  "Little by Little",
                                                  "I'm a King Bee",
                                                  "Carol",
                                                  "Tell Me",
                                                  "Can I Get a Witness",
                                                  "You Can Make It If You Try",
                                                  "Walking the Dog"};

static const char *out_of_our_heads_tracks[] = {"She Said Yeah",
                                                "Mercy, Mercy",
                                                "Hitch Hike",
                                                "That's How Strong My Love Is",
                                                "Good Times",
                                                "Gotta Get Away",
                                                "Talkin' Bout You",
                                                "Cry to Me",
                                                "Oh, Baby (We Got a Good Thing Going)",
                                                "Heart of Stone",
                                                "The Under Assistant West Coast Promotion Man",
                                                "I'm Free"};

static const char *beggars_banquet_tracks[] = {
    "Sympathy for the Devil", "No Expectations", "Dear Doctor",     "Parachute Woman", "Jig-Saw Puzzle",
    "Street Fighting Man",    "Prodigal Son",    "Stray Cat Blues", "Factory Girl",    "Salt of the Earth"};

static const char *tattoo_you_tracks[] = {"Start Me Up", "Hang Fire",         "Slave", "Little T&A", "Black Limousine",
                                          "Neighbours",  "Worried About You", "Tops",  "Heaven"};

static const char *bleach_tracks[] = {"Blew",      "Floyd The Barber", "About a Girl",   "School",
                                      "Love Buzz", "Paper Cuts",       "Negative Creep", "Scoff",
                                      "Swap Meet", "Mr. Moustache",    "Sifting"};

static const char *nevermind_tracks[] = {
    "Smells Like Teen Spirit", "In Bloom",  "Come as You Are", "Breed",     "Lithium",    "Polly",
    "Territorial Pissings",    "Drain You", "Lounge Act",      "Stay Away", "On a Plain", "Something in the Way",
    "Endless, Nameless"};

static const char *in_utero_tracks[] = {"Serve the Servants",
                                        "Scentless Apprentice",
                                        "Heart-Shaped Box",
                                        "Rape Me",
                                        "Frances Farmer Will Have Her Revenge on Seattle",
                                        "Dumb",
                                        "Very Ape",
                                        "Milk It",
                                        "Pennyroyal Tea",
                                        "Radio Friendly Unit Shifter",
                                        "tourette's",
                                        "All Apologies"};

/*! \brief Macro to determine number of elements in an array of ptrs */
#define RLEN(x) (sizeof (x) / sizeof (x[0]))

/*! \brief Main function for core06 example
 *
 * The function initializes the RDM environment and runs the create, read operations.
 *
 * @return Returns the RDM_RETCODE on exit.
 */
int main (void)
{
    RDM_DB hDB;
    RDM_RETCODE rc;

    rc = openEmptyDatabase (&hDB);
    if (rc == sOKAY)
    {
        insertArtist (hDB, "The Doors");
        insertArtist (hDB, "The Rolling Stones");
        insertArtist (hDB, "Nirvana");

        insertAlbum (hDB, "The Doors", "The Doors", the_doors_tracks, RLEN (the_doors_tracks));
        insertAlbum (hDB, "The Doors", "Strange Days", strange_days_tracks, RLEN (strange_days_tracks));
        insertAlbum (hDB, "The Doors", "Waiting for the Sun", waiting_tracks, RLEN (waiting_tracks));
        insertAlbum (hDB, "The Rolling Stones", "The Rolling Stones", the_rolling_stones_tracks,
                     RLEN (the_rolling_stones_tracks));
        insertAlbum (hDB, "The Rolling Stones", "Out of our Heads", out_of_our_heads_tracks,
                     RLEN (out_of_our_heads_tracks));
        insertAlbum (hDB, "The Rolling Stones", "Beggars Banquest", beggars_banquet_tracks,
                     RLEN (beggars_banquet_tracks));
        insertAlbum (hDB, "The Rolling Stones", "Tattoo You", tattoo_you_tracks, RLEN (tattoo_you_tracks));
        insertAlbum (hDB, "Nirvana", "Bleach", bleach_tracks, RLEN (bleach_tracks));
        insertAlbum (hDB, "Nirvana", "Nevermind", nevermind_tracks, RLEN (nevermind_tracks));
        insertAlbum (hDB, "Nirvana", "In Utero", in_utero_tracks, RLEN (in_utero_tracks));

        rc = readAllAlbums (hDB);

        cleanup (hDB);
    }
    return (int) rc;
}
