/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2011 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Birdstep LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

/** \file cpp-transaction.h
 *  \brief Header for the C++ transaction API
 */

#ifndef CPP_TRANSACTION_H
#define CPP_TRANSACTION_H

#if !defined(__cplusplus)
#error cpp-transaction.h is only for C++ applications.  Do not include it in C applications
#endif

#if defined(RDM_WINDOWS)
#pragma warning(disable : 4290)
#endif

#include "psptypes.h"
#include "rdmretcodetypes.h"
namespace RDM_CPP {
    class ITransaction;
    class Transaction;
} // namespace RDM_CPP

/*! \brief The Transaction class
   This class provides the Transaction implementation

   This is a class with very little overhead - a single pointer - that
   is a 'smart pointer' (i.e. will automatically get destroyed when the last
   reference is removed).

   To keep the overhead down this class should have no virtual functions.
  */
class RDM_PUBLIC_CLASS RDM_CPP::Transaction
{
  public:
    /* Handle more than one reference to the underlying database */

    /*! \brief Transaction Default Constructor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Transaction interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The default constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    Transaction (void) throw ();

    /*! \brief Transaction assignment operator

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Transaction interface.
        The number of references to the underlying implementation object will be
        maintained.  When the number of references to an implementation object
        is down to zero the object will be released.

        The assignment operator will increment the number of references to the
        underlying implementation object by one.
    */
    Transaction &operator= (const Transaction &transaction) throw ();

    //! Validate the ITransaction
    void _ValidateITransaction () const;

    /** \brief End a transaction by committing it

    This method commits the transaction

    \par Locking Requirements in Shared/Shared-Strict Modes
    None

    \throws
    rdm_exception (eINVTRANSID)

    \par Library
    rdmcpp-12
    */
    void End (void);

    /** \brief End a transaction by rolling it back

    This method rolls back the transaction

    \par Locking Requirements in Shared/Shared-Strict Modes
    None

    \throws
    rdm_exception (eINVTRANSID)

    \par Library
    rdmcpp-12
    */
    void EndRollback (void);

    /** \brief Rollback all changes in a transaction

    This method rolls back the transaction, but keeps the transaction active
    and any locks granted when starting the transaction remain in place.

    \par Locking Requirements in Shared/Shared-Strict Modes
    None

    \throws
    rdm_exception (eINVTRANSID)

    \par Library
    rdmcpp-12
    */
    void Rollback (void);

    /*! \internal

        \brief Release the Transaction

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Transaction interface.
        The number of references to the underlying implementation object will be
        maintained.  When the number of references to an implementation object
        is down to zero the object will be released.

        The constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    Transaction &Release (void);

    /*! \brief Transaction destructor

       The default constructor, copy constructor, assignment operator, and
       destructor implement a smart pointer for the Transaction interface.
       The number of references to the underlying implementation object will be
       maintained.  When the number of references to an implementation object
       is down to zero the object will be released.

       The destructor will decrement the number of references to the underlying
       implementation object by one.
   */
    ~Transaction ();

    /** \brief Constructor

      Applications should not directly create Transaction classes.  They should
      use the Db methods to get a Transaction class.
     */
    Transaction (
        const Transaction
            &trans /**< [in] an existing Transaction to base this instance on */
        ) throw ();

#ifndef RDM_DOXYGEN
  protected:
    ITransaction *m_itransaction;
    Transaction (ITransaction *itrans) throw ();
    RDM_RETCODE GetReturnCode () const;

  private:
    const Transaction *operator& (void) const;
#endif
    friend class Db;
};

#endif
