/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2017 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmcursorapi.h
 *  \brief Header for the RDM Cursor APIs
 */

#include "rdmtypes.h"
#include "rdmrowidtypes.h"

/**
 *
 * \defgroup cursor RDM Cursor APIs
 * \ingroup db
 *
 *    The core database API for handling and navigating data. All the
 *    functions here are located in \lib{rdm}
 *
 *    A cursor represent a set of rows from a table ordered by
 *    certain criteria.  Except where we have set relationships (\ref
 *    cursor_set) or the special cursor returned by
 *    rdm_rdmGetBeforeFirst() or rdm_rdmGetAfterLast() a cursor
 *    always represent all rows for a given table.
 *
 *    When a cursor is associated with a set of rows, the cursor will
 *    be positioned at the \e BeforeFirst position or where applicable
 *    it will be positioned at a given row or position.  Use \ref
 *    cursor_navigation to navigate forward, backward, or to a given
 *    position.  A position can be at any row in the set, between
 *    rows, or the \e BeforeFirst or \e AfterLast positions.  Use \ref
 *    cursor_position to comapre the position of two cursors.
 *
 *    The simplest cursors are singletons returned by
 *    rdm_rdmGetBeforeFirst(), rdm_rdmGetAfterLast(),
 *    rdm_cursorGetSelf(), rdm_cursorGetOwnerRow().  All other
 *    cursors are associated with zero or more rows from a given
 *    table.
 *
 * \addtogroup cursor
 * @{
 *  \defgroup cursor_cleanup Cursor Cleanup Functions
 *
 *      Cleanup API for core cursors.  All the fuctions here are
 *      located in \lib{rdm}  See \ref cursor for a more detailed
 *      description of a cursor.
 *
 *  \defgroup cursor_information Cursor Information Functions
 *
 *      API for retrieving information for core cursors.  All the
 *      fuctions here are located in \lib{rdm}  See \ref cursor
 *      for a more detailed description of a cursor.
 *
 *  \defgroup cursor_association_rowid Cursor Association Functions (rowscan)
 *
 *      Core cursor association API for retrieving rows in rowid order
 *      positioned at a given row.  All the fuctions here are located
 *      in \lib{rdm}  See \ref cursor for a more detailed
 *      description of a cursor.
 *
 *  \defgroup cursor_association_key Cursor Association Functions (keyscan)
 *
 *      Core cursor association API for retrieving rows in key order
 *      positioned at a given row.  All the fuctions here are located
 *      in \lib{rdm}  See \ref cursor for a more detailed
 *      description of a cursor.
 *
 *  \defgroup cursor_association_set Cursor Association Functions (setscan)
 *
 *      Core cursor association API for retrieving rows in a set.  All
 *      the fuctions here are located in \lib{rdm}  See \ref
 *      cursor for a more detailed description of a cursor.
 *
 *  \defgroup cursor_association_other Cursor Association Functions (other)
 *
 *      Core cursor association API for other functions.  All the
 *      fuctions here are located in \lib{rdm}  See \ref cursor
 *      for a more detailed description of a cursor.
 *
 *  \defgroup cursor_navigation Cursor Navigation Functions
 *
 *      API for navigating a core cursor.  All the fuctions here are
 *      located in \lib{rdm}  See \ref cursor for a more detailed
 *      description of a cursor.
 *
 *  \defgroup cursor_read Read Functions
 *
 *      API for reading data from a row identified by a core cursor.
 *      All the fuctions here are located in \lib{rdm}  See \ref
 *      cursor for a more detailed description of a cursor.
 *
 *  \defgroup cursor_update Update Functions
 *
 *      API for updating data for a row identified by a core cursor.
 *      All the fuctions here are located in \lib{rdm}  See \ref
 *      cursor for a more detailed description of a cursor.
 *
 *  \defgroup cursor_delete Delete Functions
 *
 *      API for deleting a row identified by a core cursor.  All the
 *      fuctions here are located in \lib{rdm}  See \ref cursor
 *      for a more detailed description of a cursor.
 *
 *  \defgroup cursor_set Set Functions
 *
 *      API for modifying set relationships for core cursors.  All the
 *      fuctions here are located in \lib{rdm}  See \ref cursor
 *      for a more detailed description of a cursor.
 *
 *  \defgroup cursor_position Cursor Comparison Functions
 *
 *      API for comparing core cursor positions.  All the fuctions
 *      here are located in \lib{rdm}  See \ref cursor for a more
 *      detailed description of a cursor.
 *
 *  \defgroup cursor_miscellaneous Miscellaneous functions
 *
 *      API for other miscellaneous core cursor functions.  All the
 *      fuctions here are located in \lib{rdm}  See \ref cursor
 *      for a more detailed description of a cursor.
 *
 * @}*/

#if defined(RDM_UNICODE)
#define rdm_cursorReadColumnIntoString                                         \
    RDM_T_FUNC (rdm_cursorReadColumnIntoString)
#define rdm_cursorUpdateColumnByString                                         \
    RDM_T_FUNC (rdm_cursorUpdateColumnByString)
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/** \brief Add a row to a set cursor (link)

    This function links the current row of \a cursor to the set associated
    with setCursor.

    \a setCursor must be a set cursor and the current row of \a cursor must be a
   valid member for the set.

    \par Locking Requirements
    Write lock on member table and read lock on owner table associated
    with the set cursor (\a setCursor).  Write lock on member table of
    the set the cursor is based on

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{eISOWNED}
    \retval{eNOTMEMBER}
    \retval{eNOTRELATED}
    \retval{ePRECOMMITTED}
    \retval{eINVCURSOROP}

    \ingroup cursor_set

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorLinkRow
    \see rdm_cursorRelinkRow
    \see rdm_cursorRemoveMember
    \see rdm_cursorUnlinkAndDeleteRow
    \see rdm_cursorUnlinkRow
    \see rdm_dbEnd
    \see rdmcompile
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorAddMember (
    RDM_CURSOR setCursor, /**< [in] The cursor whose set the row will be added to */
    RDM_CURSOR memberCursor /**< [in] The cursor whose current row will be add to the set */
);

/** \brief Close an ::RDM_CURSOR

    This function will close a cursor and free any status and row sets
    associated with the cursor. After a cursor has been closed it can continue
    to be used and associated with other row sets. Once a cursor has been freed
    with rdm_cursorFree() it can no longer be used.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_dbAllocCursor
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetClone
    \see rdm_cursorFree

    \ingroup cursor_cleanup
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorClose (
    RDM_CURSOR cursor /**< [in] The cursor to be closed  */
);

/** \brief compare the position of a cursor relative to another cursor

    This function compares the current position of \a cursorPrimary relative to
    \a cursorSecondary.

    If the cursor types are not the same the current row of the secondary will
    be used and then converted to a position in the cursor type of
    \a cursorPrimary. If the current row of the secondary is not a valid cursor
    position in the primary an error will be returned.

    This means:

    - If \a cursorPrimary is a key cursor the comparison will be based on key
    value.
    - If \a cursorPrimary is a table scan cursor the comparison will be based on
    rowid value
    - If \a cursorPrimary is a set member cursor the comparison will be based on
    the order in the set chain
    - If \a cursorPrimary is a singleton cursor the comparison is only valid if
    \a cursorSecondary and \a cursorPrimary are the same row (there is no order
   for a singleton)
    - If the row at the current position of \a cursorSecondary is not in the
    collection of \a cursorPrimary an error will be returned.
    - There are two distinct types of errors to be returned when the row in
    \a cursorSecondary is not found in the collection of \a cursorPrimary:
        - If row in \a cursorSecondary can never be in the collection of
        \a cursorPrimary based on the schema information (it is a different
   table or from another database) then ::eNOTRELATED will be returned.
        - If the row in \a cursorSecondary could be found in the collection of
        \a cursorPrimary but is not (it has a different set owner) then
        ::eNOTINCURSOR is returned.
    - In the case where the cursors are related but secondary cursor is in a
    special position (\e BeforeFirst/\e AfterLast) then the cursors must be of
   the same type to have a meaningful comparison (both key cursors based on the
    same key, both set cursors with the same owner, etc.). If it is not a
    meaningful comparison ::eNOTINCURSOR is returned.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eNOTINCURSOR}

    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorMoveToNext
    \see rdm_rdmGetBeforeFirst
    \see rdm_rdmGetAfterLast

    \ingroup cursor_position
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorComparePosition (
    RDM_CURSOR cursorPrimary,   /**< [in] The cursor whose current row is being
                                     checked relative to \a cursorSecondary */
    RDM_CURSOR cursorSecondary, /**< [in] The cursor whose current row is
                                     being checked against  */
    RDM_CURSOR_COMPARE *comparison /**< [in] A pointer to the returned
                                     comparison result value */
);

/** \brief Delete a row from a table

    This function deletes the current row of \a cursor from the database.

    After the row has been deleted the cursor will no longer have a current
    row. However it will maintain a virtual position which be the position
    between the rows neighboring the deleted row. The neighboring rows are
    dependent upon on the type of cursor.

    The ability to delete a row will be based on the triggered action specified
    in the table definition. A row that only references other rows will be
    deleted and the references automatically removed. The behavior of a row
    that is referenced by other rows will be dependent the ON DELETE triggered
    action.

    If the the triggered action is:
    - \reserved{restrict} -- an error code will be returned and no rows will be deleted
    - \reserved{set null} -- all rows that reference the row to be deleted will have
    their referencing foreign key columns set to NULL.
    - \reserved{cascade} -- all of the rows that reference the row to be deleted
    will also be deleted. If there are any referential integrity
    issues with the referencing rows an error code will be returned.

    \par Locking Requirements
    Write lock on the table the cursor is based on and potentially a
    write lock (on delete cascade) or a read lock (on delete restrict)
    on any tables that have rows that reference the row to be deleted.

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{ePRECOMMITTED}
    \retval{eREFINTEGRITY}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_dbDeleteAllRowsFromDatabase
    \see rdm_dbDeleteAllRowsFromTable
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_dbEnd

    \ingroup cursor_delete
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorDeleteRow (
    RDM_CURSOR cursor /**< [in] The cursor whose current row will be deleted */
);

/** \brief Free an ::RDM_CURSOR

    This function will free all resources associated with a cursor. Attempting
    to use a cursor after it has been freed may result in an application crash.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_cursorClose

    \ingroup cursor_cleanup
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorFree (
    RDM_CURSOR cursor /**< [in] The cursor to be freed  */
);

/** \brief Get the current size of a blob column

    This function retrieves the current size of a blob column from the
    current row of \a cursor.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_read
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetBlobSize (
    RDM_CURSOR cursor, /**< [in] The cursor to get the blob size from */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the blob column to get the size of */
    uint64_t *size /**< [out] A pointer to the returned size (in bytes) of the blob */
);

/** \brief Get the current size of a clob column

    This function retrieves the current size of a clob column from the current
    row of \a cursor.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadClob
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_read
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetClobSize (
    RDM_CURSOR cursor, /**< [in] The cursor to get the clob size from */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the clob column to get the size of */
    uint64_t *numCharacters /**< [out] A pointer to the number of characters in the clob */
);

/** \brief Associate an ::RDM_CURSOR as a clone of an existing ::RDM_CURSOR

    This function associates the ::RDM_CURSOR pointed to by \a pCursor as a clone of
    \a sourceCursor. The resulting ::RDM_CURSOR will be of the same type, contain
    the same row set, and will have the same current row as \a sourceCursor.

    \association_rules_cursor{\a sourceCursor}

    \par Locking Requirements
    Read lock on the table associated with the \a sourceCursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{sNOTFOUND}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_association_other
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetClone (
    RDM_CURSOR sourceCursor, /**< [in] The cursor to clone */
    RDM_CURSOR *pCursor      /**< [out] A pointer to an ::RDM_CURSOR (must be
                                allocated, associated      with the same db as \a
                                sourceCursor, or set to NULL) */
);

/** \brief Get the row count for a cursor

    This function returns the row count for the specified cursor.

    A singleton cursor has by definition only one row.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetCount (
    RDM_CURSOR cursor, /**< [in] The cursor to get the record count of  */
    uint64_t *count    /**< [out] A pointer to the returned record count */
);

/** \brief Get the RDM_DB handle associated with a cursor

    This function assigns the RDM_DB handle associated with an ::RDM_CURSOR to
   the \a pDb.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetDatabase (
    RDM_CURSOR cursor, /**< [in] The cursor to get the database handle for */
    RDM_DB *pDb /**< [out] A pointer to an RDM_DB that will be assigned the db
                   handle associated with cursor */
);

/** \brief Get the lock status of a table

    This function returns the current lock status of the table the cursor is
    based on. If the database handle that cursor is based on does not have a
    lock on the table then the status will be ::RDM_LOCK_FREE regardless of
    whether any other users have the table locked.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbOpen
    \see rdm_dbGetLockStatus
    \see rdm_cursorGetLockStatus
    \see rdm_dbGetTransactionStatus
    \see rdm_dbStartUpdate
    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbGetRows
    \see rdm_dbEnd
    \see rdm_transEnd

    \ingroup cursor_miscellaneous
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetLockStatus (
    RDM_CURSOR cursor,      /**< [in] A valid ::RDM_CURSOR handle */
    RDM_LOCK_STATUS *status /**< [out] The current lock status for the
                              table associated with the cursor */
);

/** \brief Get the member count for a set owner row

    This function calculates the member count of the set identified by \a refId
   for the owner identified by the current row of \a cursor.

    The current row of \a cursor must be a valid owner of \a refId.

    \par Locking Requirements
    Read lock on the member table and the owner table (\a cursor) associated
   with the set \a refId

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetSelf
    \see rdm_cursorMoveToNext

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetMemberCount (
    RDM_CURSOR cursor, /**< [in] The cursor whose current row we will get the member count of */
    RDM_REF_ID refId, /**< [in] The set identifier for the requesting count */
    uint64_t *memberCount /**< [out] A pointer to the returned set member count */
);

/** \brief Associate an ::RDM_CURSOR with members

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a set of
    rows containing all the members related to the current row of \a ownerCursor
    for the set identified by \a refId. The resulting cursor will be positioned at
    \e BeforeFirst.

    If \a ownerCursor is not positioned to a row that is in a table that is a valid
    owner of the set identified by \a refId an error will be returned and the
    ::RDM_CURSOR will not be associated.

    \a ownerCursor can be any type of cursor as long as the current row of
    \a ownerCursor is a valid owner of the set identified by \a refId.

    \association_rules_cursor{\a ownerCursor}

    \par Locking Requirements
    Read lock on the member table and the owner table (\a ownerCursor)
    associated with the set \a refId

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetSiblingRowsAtPosition
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_association_set
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetMemberRows (
    RDM_CURSOR ownerCursor, /**< [in] The cursor for the owner row */
    RDM_REF_ID refId, /**< [in] The set identifier for the requesting set of members */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with a set owner

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with the
   set owner of current row of \a memberCursor for the set identified by \a
   refId. The cursor will be positioned at this row. If the \a sourceCursor is
   not positioned to a valid member of the set identified by \a refId an error
   will be returned and the ::RDM_CURSOR will not be associated.

    \association_rules_cursor{\a memberCursor}

    \par Locking Requirements
    Read lock on the member table (\a memberCursor) and the owner table
   associated with the set \a refId

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetOwnerTableId
    \see rdm_cursorGetOwnerRowId
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRowsOfMembers
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetSiblingRowsAtPosition
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_association_set
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetOwnerRow (
    RDM_CURSOR memberCursor, /**< [in] The cursor for the member */
    RDM_REF_ID refId,   /**< [in] The set identifier for the requesting owner */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with db, or set to NULL) */
);

/** \brief Get the rowid of a set owner

    This function retrieves the rowid of the owning row of the \a refId set for
   the current record of \a cursor.

    If the current row of \a cursor is not at a valid member of the set an error
    will be returned.

    \par Locking Requirements
    Read lock on the member table and the owner table associated with
    the set \a refId

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetOwnerTableId

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetOwnerRowId (
    RDM_CURSOR cursor, /**< [in] The cursor to get the owner rowid from  */
    RDM_REF_ID refId, /**< [in] The set identifier of the set to get the owner rowid of */
    RDM_ROWID_T *rowid /**< [out] A pointer to the returned rowid */
);

/** \brief Get the table id for the \a refId owner for the current row of a cursor

    This function returns the table identifier of the \a refId owner for the row
    that is in the current position of \a cursor

    If the current row of \a cursor is not at a valid member of the set an error
    will be returned.

    \par Locking Requirements
    Read lock on member table (\a cursor) and owner table associated
    with \a refId.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetOwnerRowId
    \see rdm_cursorGetTableId

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetOwnerTableId (
    RDM_CURSOR cursor,    /**< [in] The cursor to get the table id for */
    RDM_REF_ID refId,     /**< [in] The set identifier */
    RDM_TABLE_ID *tableId /**< [out] A pointer to the returned table id */
);

/** \brief Get the rowid for the current row of the cursor

    This function retrieves the rowid for the current row of \a cursor. If the
    cursor is not at a valid row an error will be returned.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorGetOwnerRowId

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetRowId (
    RDM_CURSOR cursor, /**< [in] The cursor to get the rowid from  */
    RDM_ROWID_T *rowid /**< [out] A pointer to the returned rowid value */
);

/** \brief Get the row status information for a status cursor

    This function retrieves the current row status information for a status
    cursor.  A status cursor can be return from a bulk insert operation.

    \see rdm_dbInsertRows

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorGetStatus

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetRowStatus (
    RDM_CURSOR cursor,           /**< [in] The cursor to get the rowid from  */
    RDM_ROW_STATUS_INFO *rowStat /**< [out] A pointer to the row status value */
);

/** \brief Associate an ::RDM_CURSOR with rows from a table ordered by rowid and
           positioned to the same row as the source cursor

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   row set ordered by rowid. The row set for the ::RDM_CURSOR contains all of
   the rows for the table of \a sourceCursor. The ::RDM_CURSOR will be initially
   positioned to the same row as \a sourceCursor.

    \association_rules_cursor{\a sourceCursor}

    \par Locking Requirements
    Read lock on the table associated with the \a sourceCursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_association_rowid
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetRowsAtPosition (
    RDM_CURSOR sourceCursor, /**< [in] The cursor whose current row will be used
                                for the table and initial cursor position */
    RDM_CURSOR *pCursor      /**< [out] A pointer to an ::RDM_CURSOR (must be
                                allocated, associated with db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with a row set based on a key and
   positioned to the same row as the source cursor

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   row set ordered by the specified key. The row set contains all of the rows
   for the table that the specified key column is in. The cursor will be
   positioned to the current row of \a sourceCursor.

    If \a sourceCursor is not at a row that is a valid row of the specified key
   an error will be returned and cursor will not be associated.

    \a sourceCursor can be any type of ::RDM_CURSOR as long as the current row
   of \a sourceCursor is a valid row for the specified key.

    \association_rules_cursor{\a sourceCursor}

    \par Locking Requirements
    Read lock on the table associated with the \a sourceCursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_association_key
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetRowsByKeyAtPosition (
    RDM_CURSOR sourceCursor, /**< [in] The cursor whose current row will be used
                                for the resulting cursor position */
    RDM_KEY_ID keyId,        /**< [in] The RDM key id  */
    RDM_CURSOR *pCursor      /**< [out] A pointer to an ::RDM_CURSOR (must be
                                allocated, associated with db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR as a copy of an existing ::RDM_CURSOR with
   the rows in reverse order

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with the
    same set of rows in \a sourceCursor, but in the reverse navigation order.
    The \a pCursor can point to \a sourceCursor and in this case you will get
    \a sourceCursor back in the reverse order. The resulting cursor will be
    initially positioned to the \e BeforeFirst location, which would be the
    same as \e AfterLast of the \a sourceCursor.

    \association_rules_cursor{\a sourceCursor}

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_association_other
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetRowsInReverseOrder (
    RDM_CURSOR sourceCursor, /**< [in] The source cursor to reverse the retrieval order for */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with the current row of the \a sourceCursor

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with the
    current row of \a sourceCursor. The new cursor will be a singleton cursor
    positioned to the current row of \a sourceCursor.

    If the \a sourceCursor is not at a valid row then an error will be returned
    and the ::RDM_CURSOR will not be associated.

    \association_rules_cursor{\a sourceCursor}

    \par Locking Requirements
    Read lock on the table associated with the \a sourceCursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_association_other
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetSelf (
    RDM_CURSOR sourceCursor, /**< [in] The source cursor for the row */
    RDM_CURSOR *pCursor      /**< [out] A pointer to an ::RDM_CURSOR (must be
                                allocated, associated with db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with siblings

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a
   set of rows containing all the member rows related (as members of the same
   owner) to the current row of \a memberCursor for the set identified by \a
   refId. The resulting cursor will be positioned at \e BeforeFirst.

    If \a memberCursor is not at a row that is a valid member of the set
   identified by \a refId an error will be returned and the ::RDM_CURSOR will
   not be associated.

    \a memberCursor can be any type of ::RDM_CURSOR as long as it's current row
   is a valid member of the set identified by \a refId.

    \association_rules_cursor{\a memberCursor}

    \par Locking Requirements

    Read lock on the member table associated with the set \a refId
    (unless \a memberCursor is a set cursor associated with \a refId) and
    the table owner table associated with the set \a refId.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}

    \ingroup cursor_association_set

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_cursorGetSiblingRowsAtPosition
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetSiblingRows (
    RDM_CURSOR memberCursor, /**< [in] The source cursor for the member row */
    RDM_REF_ID
        refId, /**< [in] The set identifier for the requested set of siblings */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with db, or set to NULL) */
);

/** \brief Associate an ::RDM_CURSOR with siblings

    This function associates the ::RDM_CURSOR pointed to by \a pCursor with a set of
    rows containing all the member rows related (as members of the same owner)
    to the current row of \a memberCursor for the set identified by \a refId. The
    resulting cursor will be positioned at the same row as \a memberCursor

    If \a memberCursor is not at a row that is a valid member of the set identified
    by \a refId an error will be returned and the ::RDM_CURSOR will not be
    associated.

    \a memberCursor can be any type of ::RDM_CURSOR as long as it's current row is a
    valid member of the set identified by \a refId.

    \association_rules_cursor{\a memberCursor}

    \par Locking Requirements

    Read lock on the owner table and the member table associated with
    the set \a refId.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_association_set

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_cursorGetSiblingRows
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetOwnerRow
    \see rdm_cursorGetRowsAtPosition
    \see rdm_cursorGetRowsByKeyAtPosition
    \see rdm_cursorGetRowsInReverseOrder
    \see rdm_cursorGetSelf
    \see rdm_cursorGetClone
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetSiblingRowsAtPosition (
    RDM_CURSOR memberCursor, /**< [in] The source cursor for the member row */
    RDM_REF_ID
        refId, /**< [in] The set identifier for the requested set of siblings */
    RDM_CURSOR *pCursor /**< [out] A pointer to an ::RDM_CURSOR (must be
                           allocated, associated with db, or set to NULL) */
);

/** \brief Check a cursor for special conditions

    This function will return information about an ::RDM_CURSOR's position.

    As you navigate (using any of the rdm_cursorMove* functions) the cursor will
    typically be in any of these states: ::CURSOR_BEFORE_FIRST; ::CURSOR_AT_ROW;
    ::CURSOR_BETWEEN; or, ::CURSOR_BEFORE_FIRST. Changes to the row through this
    cursor or any other cursor may put the cursor in some other state.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorGetRowStatus

    \ingroup cursor_position
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetStatus (
    RDM_CURSOR cursor, /**< [in] The cursor whose position is being checked */
    RDM_CURSOR_STATUS *position /**< [out] The check position result value */
);

/** \brief Get the table id for the current row of the cursor

    This function returns the table identifier of the row that is in the current
    position of cursor

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorGetOwnerTableId

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetTableId (
    RDM_CURSOR cursor,    /**< [in] The cursor to get the table id for */
    RDM_TABLE_ID *tableId /**< [out] A pointer to the returned table id */
);

/** \brief Get the cursor type

    This function will return the type of cursor

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_dbGetRows
    \see rdm_dbGetRowsByKey
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf

    \ingroup cursor_information
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorGetType (
    RDM_CURSOR cursor,    /**< [in] The cursor to be freed  */
    RDM_CURSOR_TYPE *type /**< [out] The type of the cursor */
);

/** \brief Check to see if a row has members for the specified set

    This function checks to see if current row of \a cursor is a referenced
    through the set identified by \a refId.

    The cursor does not need to be a set cursor but its current row must be in a
    table that is a valid owner of the set identified by \a refId.

    The \a hasMembers parameter is set to ::RDM_TRUE if the row has members,
    otherwise ::RDM_FALSE.

    \par Locking Requirements
    Read lock on the owner and member tables associated with the set \a refId

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorHasMembers (
    RDM_CURSOR cursor,     /**< [in] The cursor to check set ownership status */
    RDM_REF_ID refId,      /**< [in] The set identifier for the request */
    RDM_BOOL_T *hasMembers /**< [in] A pointer to the referenced status */
);

/** \brief Check to see if a row is a member for the specified set

    This function checks to see if current row of \a cursor is an active set
    member (is linked to an owner) for the set identified by \a refId.

    The cursor does not need to be a set cursor but its current row must be in
    a table that is a valid member of the set identified by \a refId.

    The \a hasOwner parameter is set to ::RDM_TRUE if the record has an active
   owner, otherwise ::RDM_FALSE.

    \par Locking Requirements
    Read lock on the member table associated with the set \a refId

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext

    \ingroup cursor_information
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorHasOwner (
    RDM_CURSOR cursor,   /**< [in] The cursor to check set ownership status */
    RDM_REF_ID refId,    /**< [in] The set identifier for the request */
    RDM_BOOL_T *hasOwner /**< [in] A pointer to the returned status */
);

/** \brief Check to see if a cursor is positioned after the last row in a collection

    This function will set \a isAfterLast to ::RDM_TRUE if the cursor is positioned
    after the last row in the cursor collection.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToAfterLast

    \ingroup cursor_position
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorIsAfterLast (
    RDM_CURSOR cursor, /**< [in] The cursor whose position is being checked */
    RDM_BOOL_T *isAfterLast /**< [out] A pointer to the returned condition */
);

/** \brief Check to see if a cursor is positioned before the first row in a collection

    This function will set \a isBeforeFirst to ::RDM_TRUE if the cursor
    is positioned before the first row in the cursor collection.

    \par Locking Requirements
    None

    \retval{sOKAY}
    \retval{eDBNOTOPEN}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorGetMemberRows
    \see rdm_cursorGetSelf
    \see rdm_cursorGetSiblingRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToBeforeFirst

    \ingroup cursor_position
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorIsBeforeFirst (
    RDM_CURSOR cursor, /**< [in] The cursor whose position is being checked */
    RDM_BOOL_T *isBeforeFirst /**< [out] A pointer to the returned condition */
);

/** \brief Link a row to an owner

    This function links the current row of \a cursor to the current row of
    \a cursorOwner for the set identified by \a refId.

    \a cursor does not need to be a set cursor, but must be a cursor whose
   current row is a valid member of \a refId

    \a cursorOwner does not need to be a set cursor, but must be a cursor whose
    current row is a valid owner of \a refId

    \par Locking Requirements
    Write lock on member table (\a cursor) and read lock on owner table
    (\a cursorOwner) associated with \a refId.

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{eISOWNED}
    \retval{eNOTOWNER}
    \retval{eNOTMEMBER}

    \ingroup cursor_set

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorAddMember
    \see rdm_cursorRelinkRow
    \see rdm_cursorRemoveMember
    \see rdm_cursorUnlinkAndDeleteRow
    \see rdm_cursorUnlinkRow
    \see rdm_dbEnd
    \see rdmcompile
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorLinkRow (
    RDM_CURSOR cursor,     /**< [in] The row to link */
    RDM_REF_ID refId,      /**< [in] The set identifier for the connection */
    RDM_CURSOR cursorOwner /**< [in] The owner to link to */
);

/** \brief Position a cursor to the \e AfterLast position

    This function will position a cursor to the \e AfterLast position.

    \par Locking Requirements
    If \a cursor is a set cursor then the owner table associated
    need a read lock.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToAfterLast (
    RDM_CURSOR cursor /**< [in] The cursor to be positioned */
);

/** \brief Position a cursor to the \e BeforeFirst position

    This function will position a cursor to the \e BeforeFirst position.

    \par Locking Requirements
    If \a cursor is a set cursor then the owner table associated
    need a read lock.

    \retval{sOKAY}
    \retval{eDBNOTOPEN}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToBeforeFirst (
    RDM_CURSOR cursor /**< [in] The cursor to be positioned */
);

/** \brief Position a cursor to the first row in the collection

    This function will position a cursor to the first row in the collection.
    If there are no rows in the collection then the cursor will be positioned
    to \e AfterLast and the function will return ::sENDOFCURSOR.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated with that cursor also
    need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{sENDOFCURSOR} No rows in the collection.
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToFirst (
    RDM_CURSOR cursor /**< [in] The cursor to be positioned */
);

/** \brief Position a cursor based on a key value

    This function will position a cursor to the first row in the row set that
    has the specified key value. The cursor to be positioned can be any type
    of cursor and does not need to be a key scan cursor based on keyId.
    Using the partial key allows a subset of the columns in a compound key to be
    considered in the search.

    If there is no exact match then:

    - If the cursor is a key scan cursor based on the same key as the specified
      key then the function will return ::sNOTFOUND and the cursor will be
   placed in the ::CURSOR_BETWEEN position. Navigating forward
   (rdm_cursorMoveToNext()) from that position will position the cursor at the
   next key value larger than the specified key value. Navigating backward
   (rdm_cursorMoveToPrevious()) from that position will position the cursor at
   the next key value smaller than the specified key value (assuming the cursor
   is not in reverse order).
    - Otherwise the function will return ::eNOTINCURSOR and the cursor position
      will be left unchanged.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the keyValue.value buffer is smaller than the size required
    to fully define the key then the remainder of the full buffer will be padded
    with NULL bytes and the search will be performed using that value.
    - If the size of the keyValue.value buffer is larger than the size required
    to fully define the key then the extra bytes will be ignored and the search
    will be performed using the truncated value.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated also need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{sNOTFOUND} Did not find an exact match, but positioned cursor
   between keys. \retval{eNOTINCURSOR}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToKey (
    RDM_CURSOR cursor,    /**< [in] The cursor to be positioned */
    RDM_KEY_ID keyId,     /**< [in] The key id */
    const void *keyValue, /**< [in] The key value to position to */
    size_t bytesIn        /**< [in] The length of keyValue in bytes */
);

/** \brief Position a cursor to the last row in the collection

    This function will position a cursor to the last row in the collection.
    If there are no rows in the collection then the cursor will be positioned
    to \e BeforeFirst and the function will return ::sENDOFCURSOR.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated also need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{sENDOFCURSOR} No rows in the collection.

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToLast (
    RDM_CURSOR cursor /**< [in] The cursor to be positioned */
);

/** \brief Position a cursor to the next row in the collection

    This function will position a cursor to the next row in the collection.

    If cursor was positioned to the last row in the collection then the cursor
    will be positioned to \e AfterLast and the function will return the status
    ::sENDOFCURSOR.

    If cursor was positioned to \e AfterLast then the function will return the
    error ::eINVITERATION.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated also need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{sENDOFCURSOR} Reached the end of the collection.
    \retval{eINVITERATION}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToNext (
    RDM_CURSOR cursor /**< [in] The cursor to be positioned */
);

/** \brief Position a cursor based on the currow row of another cursor

    This function will position a cursor to the current row of another cursor.
    If the row is not in the row set an error (::eNOTINCURSOR) will be
    returned and the cursor position will be left unchanged.

    \note If the exact cursor position and not the current row is wanted (the two
    cursors have to be of the same type) one would need to use
    rdm_cursorGetClone() instead.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated also need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{sNOTFOUND} Did not find an exact match, but positioned cursor between records.
    \retval{eNOTINCURSOR}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToPosition (
    RDM_CURSOR cursor1, /**< [in] The cursor to be positioned */
    RDM_CURSOR cursor2 /**< [in] The cursor whose current row cursor1 will be positioned to */
);

/** \brief Position a cursor to the previous row in the collection

    This function will position a cursor to the previous row in the collection.

    If cursor was positioned to the first row in the collection then the cursor
    will be positioned to \e BeforeFirst and the function will return the status
    ::sENDOFCURSOR.

    If cursor was positioned to \e BeforeFirst then the function will return the
    error ::eINVITERATION.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated also need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{sENDOFCURSOR} Reached the end of the collection.
    \retval{eINVITERATION}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToRowId
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToPrevious (
    RDM_CURSOR cursor /**< [in] The cursor to be positioned */
);

/** \brief Position a cursor based on a rowid

    This function will position a cursor to the row at specified rowid. The
    cursor can be any type of cursor and does not need to be a table scan
    cursor.

    If there is no exact match found then:

    - If the cursor is a table scan cursor then the function will return
      ::sNOTFOUND and the cursor will be placed in the ::CURSOR_BETWEEN
      position. Navigating forward (rdm_cursorMoveToNext()) from that
      position will place the cursor at the next valid rowid larger than the
      specified rowid. Navigating backward (rdm_cursorMoveToPrevious()) from
      that position will position the cursor at the next rowid smaller than the
      specified rowid (assuming the cursor is not in reverse order).

    - Otherwise the function will return ::eNOTINCURSOR and the cursor
    position will be left unchanged.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated also need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{eNOTINCURSOR}
    \retval{sNOTFOUND}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToSearchKey
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToRowId (
    RDM_CURSOR cursor, /**< [in] The cursor to be positioned */
    RDM_ROWID_T rowid  /**< [in] The rowid to position the cursor to */
);

/** \brief Position a cursor based on a key value

    This function will position a cursor to the first row in the row set that
    has the specified key value. The cursor to be positioned can be any type
    of cursor and does not need to be a key scan cursor based on \a keyId.
    Using the partial key allows a subset of the columns in a compound key to be
    considered in the search.

    If there is no exact match then:

    - If the cursor is a key scan cursor based on the same key as the specified
      key then the function will return ::sNOTFOUND and the cursor will be
   placed in the ::CURSOR_BETWEEN position. Navigating forward (rdm_cursorMoveToNext())
   from that position will position the cursor at the next key value larger than
   the specified key value. Navigating backward (rdm_cursorMoveToPrevious())
   from that position will position the cursor at the next key value smaller
   than the specified key value (assuming the cursor is not in reverse order).
    - Otherwise the function will return ::eNOTINCURSOR and the cursor
      position will be left unchanged.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \c{keyValue.value} buffer is smaller than the size
   required to fully define the key then the remainder of the full buffer will
   be padded with NULL bytes and the search will be performed using that value.
    - If the size of the \c{keyValue.value} buffer is larger than the size
   required to fully define the key then the extra bytes will be ignored and the
   search will be performed using the truncated value.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor.  If \a cursor is
    a set cursor then the owner table associated also need a read lock.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{sNOTFOUND} Did not find an exact match, but positioned cursor
   between keys. \retval{eNOTINCURSOR}

    \see rdm_dbStartRead
    \see rdm_dbStartSnapshot
    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToAfterLast
    \see rdm_cursorMoveToBeforeFirst
    \see rdm_cursorMoveToFirst
    \see rdm_cursorMoveToKey
    \see rdm_cursorMoveToLast
    \see rdm_cursorMoveToNext
    \see rdm_cursorMoveToPosition
    \see rdm_cursorMoveToPrevious
    \see rdm_cursorMoveToRowId
    \see rdm_cursorComparePosition
    \see rdm_cursorReadRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow

    \ingroup cursor_navigation
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorMoveToSearchKey (
    RDM_CURSOR cursor,             /**< [in] The cursor to be positioned */
    RDM_KEY_ID keyId,              /**< [in] The key id */
    const RDM_SEARCH_KEY *keyValue /**< [in] The key value to position to */
);

/** \brief Read data from a blob column

    This function reads contents of a blob column from the current row of
    cursor.

    The read will start from the \a offset position and will attempt to read up
   to \a bytesIn bytes. If there are not \a bytesIn bytes left to read and the
    \a bytesOut parameter is NULL then the function will return the
   ::eBLOBBADSIZE error code.

    If \a bytesOut is not NULL then the number of bytes actually read into value
    will be put into that parameter.

    If value is NULL and \a bytesOut is not NULL then \a bytesOut will have the
    remaining number of bytes to be read from the \a offset position.

    \par Locking Requirements
    Read lock on the tables associated with the \a cursor.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorGetBlobSize
    \see rdm_cursorReadClob
    \see rdm_cursorReadRow
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile

 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorReadBlob (
    RDM_CURSOR cursor, /**< [in] The cursor to read data from */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the blob column to read */
    uint64_t offset, /**< [in] Offset into the blob to begin reading */
    void *value, /**< [in] A pointer to the buffer where the the column data will be placed */
    size_t bytesIn, /**< [in] The length the the value buffer (in bytes for both CLOB and BLOB columns) */
    size_t *bytesOut /**< [out] A pointer to the number of bytes actually read */
);

/** \brief Read data from a clob column as UTF-8

    This function reads contents of a clob column from the current row of
    cursor.

    The read will start from the \a offset position (in characters) and will
    attempt to read up to \a bytesIn bytes. If there are not \a bytesIn bytes left to
    read and the \a bytesOut parameter is NULL then the function will return the
    ::eBLOBBADSIZE error code.

    If \a bytesOut is not NULL then the number of bytes (as UTF-8) actually read
    into value will be put into that parameter.

    If \a charactersOut is not NULL then the number of characters actually read
    into value will be put into that parameter.

    If value is NULL and \a bytesOut is not NULL then \a bytesOut will have the
    remaining number of bytes (as UTF-8) to be read from the \a offset position.

    For a CLOB column you cannot assume that you have reached the end of the
    column if \c{bytesOut != bytesIn}. It is possible that there were not enough
    bytes left in value for a complete character. A CLOB column should not
    be considered completely read until the \a bytesOut or \a charactersOut parameter
    returns 0. An error will be returned if the size of value is not large
    enough to hold a single character from a CLOB column.

    \par Locking Requirements
    Read lock on the tables associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_read
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorReadClob (
    RDM_CURSOR cursor, /**< [in] The cursor to read data from */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the clob column to read */
    uint64_t offset, /**< [in] Offset (in characters) into the blob to begin reading */
    char *value, /**< [in] A pointer to the buffer where the column data will be placed */
    size_t bytesIn, /**< [in] The length the value buffer (in bytes) */
    size_t *bytesOut, /**< [out] A pointer to the number of bytes (as UTF-8) actually read */
    size_t *charactersOut /**< [out] A pointer to the number of characters actually read */
);

/** \brief Read a single column from a table row

    This function reads the value of a single column from the current row of
    cursor into the \a columnValue buffer in the format specified by the schema.

    If the column is NULL the status code ::sNULLVAL will be returned.

    If \a columnValue is NULL, \a bytesIn is 0, and \a bytesOut is not NULL then the
    size of the column will be put into \a bytesOut. This will allow the
    application to allocate a buffer large enough to read the column value.

    If the column is a blob an attempt to read the entire blob into the
    specified buffer will be made. If \a bytesIn is not large enough to hold the
    entire blob column a status code (::sTRUNCATE) will be returned. If you
    want to read a portion of the blob then use rdm_cursorReadBlob() or
    rdm_cursorReadClob() as these will allow for specifying the \a offset to
    start reading.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a columnValue buffer is smaller than the size of the
    column then the function will write as much column data as possible into
    the buffer and return a ::sTRUNCATE status code. There is no
    guarantee the contents of a truncated column will have valid/useful data
    (or be NULL terminiated if the column is a string).
    - If the size of the \a columnValue buffer is larger than the column size then
    the function will write the full column value and return ::sOKAY.

    The \a bytesOut parameter can be used to determine how much data
    was read into the buffer.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor. A foreign key also require
    read lock on the referenced table.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{sNULLVAL} The column is NULL.
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_read

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadColumnIntoString
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorReadColumn (
    RDM_CURSOR cursor,      /**< [in] The cursor to read data from */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the column to read */
    void *columnValue, /**< [in] A pointer to the buffer where the column value
                         will be read */
    size_t bytesIn, /**< [in] The length of the \a columnValue buffer (in bytes) */
    size_t *bytesOut /**< [out] A pointer to the number of bytes/characters
                           actually read (can be NULL) */
);

/** \brief Read and convert a single column to a string

    This function reads the value of a single column from the current row of
    cursor and converts it into a string.

    If the column is NULL the status code ::sNULLVAL will be returned.

    If \a columnValue is NULL, inBytes is 0, and \a bytesOut is not NULL then
   the size of the column will be put into \a bytesOut. This will allow the
   application to allocate a buffer large enough to read the column value.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a columnValue buffer is smaller than the size of the
    type specified required when doing the conversion then the function will
    place as much data as possible into the buffer and return a
    ::sTRUNCATE status code. The returned string will be NULL terminated.
    - If the size of the \a columnValue buffer is larger than the size required
    for the conversion the function will write the full column value and
    return sOKAY.

    The \a bytesOut parameter can be used to determine how much data could be
   read into the buffer.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor. A foreign key also
   require read lock on the referenced table.

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{sNULLVAL} The column is NULL.
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_read

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadColumn
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorReadColumnIntoString (
    RDM_CURSOR cursor,      /**< [in] The cursor to read data from */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the column to read */
    RDM_TCHAR_T *columnValue, /**< [in] A pointer to the buffer where the
                         converted column value will be read */
    size_t inBytes, /**< [in] The length of the \a columnValue buffer in bytes */
    size_t *bytesOut /**< [out] A pointer to the number of bytes actually read
                       (can be NULL) */
);

/** \brief Read the columns for a key from a table row

    This function reads the value of all columns in a key the current row of
    cursor into the keyValue buffer in the format specified by the schema.

    If the key is NULL the status code ::sNULLVAL will be returned.

    If keyValue is NULL, \a bytesIn is 0, and \a bytesOut is not NULL then the
    size of the key will be put into \a bytesOut. This will allow the
    application to allocate a buffer large enough to read the key value.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the keyValue buffer is smaller than the size of the
    key then the function will write as much key data as possible into
    the buffer and return a ::sTRUNCATE status code. There is no
    guarantee the contents of a truncated column will have valid/useful data
    (or be NULL terminiated if the column is a string).
    - If the size of the keyValue buffer is larger than the key size then
    the function will write the full key value and return sOKAY.

    The \a bytesOut parameter can be used to determine how much data
    was read into the buffer.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{sNULLVAL} The column is NULL.
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_read

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadRow
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorReadKey (
    RDM_CURSOR cursor, /**< [in] The cursor to read data from */
    RDM_KEY_ID keyId,  /**< [in] The key identifier of the key to read */
    void *keyValue,    /**< [in] A pointer to the buffer where the key value
                            will be read */
    size_t bytesIn,    /**< [in] The length of the keyValue buffer (in bytes) */
    size_t *bytesOut   /**< [out] A pointer to the number of bytes/characters
                             actually read (can be NULL) */
);

/** \brief Read all columns from a row

    This function reads all the (non blob) column values from the current row
    of cursor.

    For each column that is not designated as NOT NULL a \c{_has_value} field is
    added to the table structure. If the \c{_has_value} field for a column is
   not set the value for the column will be considered NULL and anything in the
    structure value field for the column will be ignored.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a colValues buffer is smaller than the row size of
    the table then the function will place as much data as possible into the
    buffer and return a ::sTRUNCATE status code. There is no guarantee
    the contents of a truncated row will have valid/useful data (or be NULL
    terminiated if the column is a string).
    - If the size of the \a colValues buffer is larger than the size required
    for the row the function will write the full column value and return sOKAY.

    The \a bytesOut parameter can be used to determine how many bytes were
    actually read into the buffer.

    \par Locking Requirements
    Read lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eNOSTARTREAD}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eCATMISMATCH} The database definition does not match what is
   expected for this application. \retval{eNOCURRENTROW} \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_read

    \see rdm_dbStartRead
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorReadBlob
    \see rdm_cursorReadColumn
    \see rdm_cursorReadKey
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_cursorLinkRow
    \see rdm_dbEnd
    \see rdmcompile
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorReadRow (
    RDM_CURSOR cursor, /**< [in] The cursor to read data from */
    void *colValues, /**< [in] A pointer to the buffer where row values will be read */
    size_t bytesIn, /**< [in] The length of the \a colValues buffer (in bytes) */
    size_t *bytesOut /**< [out] A pointer to the number of bytes actually read (can be NULL) */
);

/** \brief Unlink a row from one owner and link to another

    This function links the current row of \a cursor to the current row of
    \a cursorOwner for the set identified by \a refId. If the row is already
    linked to a different owner it will be unlinked from that owner before
    being linked to the new owner.

    \a cursor does not need to be a set cursor, but it must be a cursor
    whose current row is a valid member of \a refId

    \a cursorOwner does not need to be a set cursor, but must be a cursor whose
    current row is a valid owner of \a refId

    If cursor is a set cursor based on \a refId the position of the cursor will
    remain at the same location even though the row is no longer in the set
    chain. This is considered the virtual position ::CURSOR_UNLINKED. It is not
    possible to read or write to a record when we are in this virtual position,
    but navigation is possible and will act as if we are navigating from the
    original cursor position.

    \par Locking Requirements
    Write lock on member table (\a cursor) and read lock on owner table
    (\a cursorOwner) associated with \a refId.

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{eNOTLINKED}
    \retval{eNOTOWNER}
    \retval{eNOTMEMBER}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorAddMember
    \see rdm_cursorLinkRow
    \see rdm_cursorRemoveMember
    \see rdm_cursorUnlinkAndDeleteRow
    \see rdm_cursorUnlinkRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_set
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorRelinkRow (
    RDM_CURSOR cursor,     /**< [in] The record to relink */
    RDM_REF_ID refId,      /**< [in] The set identifer for the link  */
    RDM_CURSOR cursorOwner /**< [in] The new owner to link to */
);

/** \brief Remove a row from a set cursor (unlink)

    This function unlinks the current row of setCursor from the set
    associated with setCursor.

    setCursor must be a set cursor

    The setCursor will maintain the origin current row and position even though
    the current row is no longer in the set. Calling rdm_cursorMoveToNext()
    or rdm_cursorMoveToPrevious() will position the cursor relative to the
    original position.

    \par Locking Requirements
    Write lock on member table and read lock on owner table associated
    with the \a setCursor.

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{eINVCURSOROP}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorAddMember
    \see rdm_cursorLinkRow
    \see rdm_cursorRelinkRow
    \see rdm_cursorUnlinkAndDeleteRow
    \see rdm_cursorUnlinkRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_set

*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorRemoveMember (
    RDM_CURSOR setCursor /**< [in] The record we want to remove */
);

/** \brief Set the size of a blob column

    This function updates the size of a blob column in current row of \a cursor.

    If size is smaller than the current size of the blob existing data will be
    truncated.

    If size is larger than the current size of the blob the blob will be padded
    with NULL bytes to extend the size of the blob.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateBlob
    \see rdm_cursorSetClobSize
    \see rdm_cursorUpdateRow
    \see rdm_cursorUpdateBlob
    \see rdm_cursorUpdateColumn
    \see rdm_cursorLinkRow
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_update
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorSetBlobSize (
    RDM_CURSOR cursor, /**< [in] The cursor containing the row with the blob to change the size of */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the blob column to change the size of */
    uint64_t size /**< [in] The new size of the blob column (in bytes) */
);

/** \brief Set the size of a clob column

    This function updates the size of a clob column in current row of \a cursor.

    The size of a clob is specified in characters.

    If \a size is smaller than the current size of the clob existing data will
   be truncated.

    If \a size is larger than the current size of the clob the clob will be
   padded with NULL characters to extend the size of the clob.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateClob
    \see rdm_cursorSetBlobSize
    \see rdm_cursorUpdateRow
    \see rdm_cursorUpdateBlob
    \see rdm_cursorUpdateColumn
    \see rdm_cursorLinkRow
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_update
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorSetClobSize (
    RDM_CURSOR cursor, /**< [in] The cursor containing the row with the clob to change the size of */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the clob column to change the size of */
    uint64_t size /**< [in] The new size of the clob column (in characters) */
);

/** \brief Unlink a row from all sets and delete it

    This function unlinks the current row of \a cursor from all sets and
    deletes it.

    If \a cursor is a set cursor based on the specified \a refId then the cursor
   will maintain the original current row and position even though the row is no
    longer in the set. Calling rdm_cursorMoveToNext() or
    rdm_cursorMoveToPrevious() will position the cursor relative to the
    original position.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor and any other
    table needed for unlinking the row from those sets.

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorAddMember
    \see rdm_cursorLinkRow
    \see rdm_cursorRelinkRow
    \see rdm_cursorRemoveMember
    \see rdm_cursorUnlinkAndDeleteRow
    \see rdm_cursorUnlinkRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_set
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorUnlinkAndDeleteRow (
    RDM_CURSOR cursor /**< [in] The row to unlink and delete */
);

/** \brief Unlink a row from its owner

    This function unlinks the current row of \a cursor from the owner for set
    identified by \a refId.

    If \a cursor is a set cursor based on the specified \a refId then the cursor
   will maintain the original current row and position even though the row is no
    longer in the set. Calling rdm_cursorMoveToNext() or
    rdm_cursorMoveToPrevious() will position the cursor relative to the
    original position.

    \par Locking Requirements
    Write lock on member table (\a cursor) and read lock on owner table
    associated with \a refId.

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}
    \retval{eNOTLINKED}
    \retval{eNOTMEMBER}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorAddMember
    \see rdm_cursorLinkRow
    \see rdm_cursorRelinkRow
    \see rdm_cursorRemoveMember
    \see rdm_cursorUnlinkAndDeleteRow
    \see rdm_cursorUnlinkRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_set
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorUnlinkRow (
    RDM_CURSOR cursor, /**< [in] The row to unlink */
    RDM_REF_ID refId   /**< [in] The set identifier for the connection */
);

/** \brief Update a blob column

    This function updates the value of a blob column in current row of \a cursor.

    If \a offset is smaller than the current size of the blob existing data will
    be overwritten.

    If \a offset is larger than the current size of the blob the blob will be
    padded with NULL bytes to extend the size of the blob to \a offset.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorUpdateClob
    \see rdm_cursorUpdateColumn
    \see rdm_cursorLinkRow
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_update
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorUpdateBlob (
    RDM_CURSOR cursor, /**< [in] The cursor containing the row with the blob column to be updated */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the blob column to be updated  */
    uint64_t offset, /**< [in] The offset into the blob to write the updated content (in bytes) */
    const void *value, /**< [in] A pointer to the content to be written to the blob column */
    size_t bytesIn /**< [in] The length of the \a value buffer */
);

/** \brief Update a clob column

    This function updates the value of a clob column in current row of \a cursor.

    If \a offset is smaller than the current size of the clob existing data will
    be overwritten.

    If \a offset is larger than the current size of the clob the clob will be
    padded with NULL characters to extend the size of the clob to \a offset.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor

    \retval{sOKAY}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorUpdateBlob
    \see rdm_cursorUpdateColumn
    \see rdm_cursorLinkRow
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbEnd
    \see rdmcompile

    \ingroup cursor_update
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorUpdateClob (
    RDM_CURSOR cursor, /**< [in] The cursor containing the row with the clob column to be updated */
    RDM_COLUMN_ID columnId, /**< [in] The column identifier of the clob column to be updated  */
    uint64_t offset, /**< [in] The character offset into the clob to write the updated content */
    const char *value, /**< [in] A pointer to the content to be written to the clob column */
    size_t charactersIn /**< [in] The number of characters in the \a value buffer */
);

/** \brief Update a single column in a row

    This function updates the value of a single column in the current
    row of \a cursor.

    If \a columnValue is NULL then \a bytesIn will be ignored and the engine will
    attempt to set the specified column to the NULL value.

    If the cursor is a key scan cursor based on the column that is
    updated then the cursor will maintain the original position but
    current row is kept the same. Calling rdm_cursorMoveToNext() or
    rdm_cursorMoveToPrevious() will position the cursor relative to the
    original value and not the updated value. The updated value will be in the
    collection.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a columnValue buffer is smaller than the size required to
    fully define the column then the ::eBADDATALEN error code will be
    returned and no update will be made to the table.
    - If the size of the buffer is larger than the size required to fully define
    a row then the extra bytes will be ignored and no status will be returned.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor. A foreign key also require
    read lock on the referenced table.

    \retval{sOKAY}
    \retval{eDUPLICATE}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_update

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateColumnByString
    \see rdm_cursorUpdateRow
    \see rdm_cursorUpdateBlob
    \see rdm_cursorUpdateColumn
    \see rdm_cursorLinkRow
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbEnd
    \see rdmcompile
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorUpdateColumn (
    RDM_CURSOR cursor, /**< [in] The cursor containing the row to be updated */
    RDM_COLUMN_ID columnId, /**< [in] The columns identifier of the column to be updated */
    const void *columnValue, /**< [in] A pointer to the buffer containing the new column value */
    size_t bytesIn /**< [in] The length of the \a columnValue buffer (in bytes) */
);

/** \brief Update a single column in a row from a string

    This function updates the value of a single column in the current row of
    cursor. This version will convert the value from a string to the
    schema type for the column.

    If \a columnValue is NULL then the type and \a bytesIn will be ignored and the
    engine will attempt to set the specified column to the NULL value.

    If the cursor is a key scan cursor based on the column that is
    updated then the cursor will maintain the original position but
    current row is kept the same. Calling rdm_cursorMoveToNext() or
    rdm_cursorMoveToPrevious() will position the cursor relative to the
    original value and not the updated value. The updated value will be in the
    collection.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a columnValue buffer is smaller than the size required for
    the specified type then the S_BUFFER_TOO_SMALL error code will be
    returned and no update will be made to the row.
    - If the size of the buffer is larger than the size required for the
    specified type then the extra bytes will be ignored and no status will be
    returned.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor. A foreign key also require
    read lock on the referenced table.

    \retval{sOKAY}
    \retval{eDUPLICATE}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eNOCURRENTROW}
    \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_update

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorUpdateBlob
    \see rdm_cursorUpdateColumn
    \see rdm_cursorLinkRow
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbEnd
    \see rdmcompile

*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorUpdateColumnByString (
    RDM_CURSOR cursor, /**< [in] The cursor containing the row to be updated */
    RDM_COLUMN_ID columnId, /**< [in] The columns identifier of the column to be updated */
    const RDM_TCHAR_T *
        columnValue, /**< [in] A pointer to the buffer containing the new column value */
    size_t bytesIn /**< [in] The length of the \a columnValue buffer */
);

/** \brief Update all columns in a row

    This function updates all column values, except for blobs, in current
    row of \a cursor.

    For each column that is not designated as NOT NULL a \c{_has_value} field is
    added to the table structure. If the \c{_has_value} field for a column is
   not set the value for the column will be considered NULL and anything in the
    structure value field for the column will be ignored.

    The engine validates the size of the buffers prior to accessing them.
    - If the size of the \a columnValue buffer is smaller than the size required
    to fully define a row then the ::eBADDATALEN error code will be
    returned and no updates to the row will be made
    - If the size of the buffer is larger than the size required to
    fully define a row then the extra bytes will be ignored and the column
    values will be updated.

    If the cursor is a key scan cursor then the cursor will maintain
    the original position but current row is kept the same.
    Calling rdm_cursorMoveToNext() or rdm_cursorMoveToPrevious() will
    position the cursor relative to the original position and not the updated
    position. The updated position will be in the collection.

    \par Locking Requirements
    Write lock on the table associated with the \a cursor. Foreign keys also
   require read locks on the referenced tables.

    \retval{sOKAY}
    \retval{eDUPLICATE}
    \retval{eREADONLY}
    \retval{eNOSTARTUPDATE}
    \retval{eNOTLOCKED}
    \retval{eDBNOTOPEN}
    \retval{eCATMISMATCH} The database definition does not match what is
   expected for this application. \retval{eNOCURRENTROW} \retval{eOWNERDELETED}
    \retval{eSINGLETONDELETED}

    \ingroup cursor_update

    \see rdm_dbStartUpdate
    \see rdm_dbGetRows
    \see rdm_cursorMoveToNext
    \see rdm_cursorUpdateRow
    \see rdm_cursorUpdateBlob
    \see rdm_cursorUpdateColumn
    \see rdm_cursorLinkRow
    \see rdm_dbInsertRow
    \see rdm_cursorUpdateRow
    \see rdm_cursorDeleteRow
    \see rdm_dbEnd
    \see rdmcompile
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_cursorUpdateRow (
    RDM_CURSOR cursor, /**< [in] The cursor containing the row to be updated */
    const void *colValues, /**< [in] A buffer that contains the column values of
                             the row to be inserted formatted as the table
                             structure generated when the database schema was
                             compiled. If the table definition has changed
                             since the schema was first compiled a new header
                             file will need to be generated. */
    size_t bytesIn /**< [in] The length of the \a colValues buffer. If \a
                     colValues is NULL this value should be 0. */
);

#if defined(__cplusplus)
}
#endif
